package ru.indvdum.mywork.vaadin;

import static ru.indvdum.mywork.MyWork.EM;

import java.util.Calendar;
import java.util.List;

import javax.persistence.Query;

import ru.indvdum.mywork.openjpa.model.Day;
import ru.indvdum.mywork.openjpa.model.Task;
import ru.indvdum.mywork.openjpa.model.Work;

import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.Property.ValueChangeListener;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.RichTextArea;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.AbstractSelect.Filtering;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;

public class EditWorkDialog extends Window implements ClickListener {
	
	private MyWorkApplication myApp = null;
	private EditState state = null;
	private Work work = null;
	private ComboBox workDay = null;
	private ComboBox workTask = null;
	private TextField workHours = null;
	private RichTextArea workResult = null;

	public EditWorkDialog(MyWorkApplication myApp, Work work) throws Exception {
		super();
		this.myApp = myApp;
		if(work == null)
			state = EditState.CREATE;
		else
			state = EditState.EDIT;
		switch(state){
		case CREATE:
			setCaption("Creating new work");
			work = new Work();
			work.setHours(0f);
			work.setResult("");
			this.work = work;
			break;
		case EDIT:
			setCaption("Editing work");
			this.work = work;
			break;
		}
		init();
	}
	
	private void init(){
		setModal(true);
		setWidth("640px");
		setHeight("480px");
		VerticalLayout vl = new VerticalLayout();
		vl.setSizeFull();
		vl.setSpacing(true);
		vl.setMargin(true);
		setContent(vl);
		
		HorizontalLayout hlDay = new HorizontalLayout();
		hlDay.setWidth("100%");
		Label lblDay = new Label("Day:");
		lblDay.setWidth("100%");
		hlDay.addComponent(lblDay);
		hlDay.setExpandRatio(lblDay, 1f);
		workDay = new ComboBox();
		workDay.setWidth("100%");
		workDay.setImmediate(true);
		workDay.setFilteringMode(Filtering.FILTERINGMODE_CONTAINS);
		updateDays("select d from " + Day.class.getName() + " d order by d.day");
		workDay.addListener(new ValueChangeListener(){

			@Override
			public void valueChange(ValueChangeEvent event) {
				Day day = (Day)event.getProperty().getValue();
				if(day == null)
					updateTasks("select t from " + Task.class.getName() + " t order by t.id desc");
				else
					updateTasks("select t from " + Task.class.getName() + " t where t.id not in (select w.task.id from " + Work.class.getName() + " w where w.day.id = " + day.getId() + ") order by t.id desc");
			}
			
		});
		hlDay.addComponent(workDay);
		hlDay.setExpandRatio(workDay, 5f);
		vl.addComponent(hlDay);
		
		HorizontalLayout hlTask = new HorizontalLayout();
		hlTask.setWidth("100%");
		Label lblStart = new Label("Task:");
		lblStart.setWidth("100%");
		hlTask.addComponent(lblStart);
		hlTask.setExpandRatio(lblStart, 1f);
		workTask = new ComboBox();
		workTask.setWidth("100%");
		workTask.setImmediate(true);
		workTask.setFilteringMode(Filtering.FILTERINGMODE_CONTAINS);
		Day day = (Day)workDay.getValue();
		if(day == null)
			updateTasks("select t from " + Task.class.getName() + " t order by t.id desc");
		else
			updateTasks("select t from " + Task.class.getName() + " t where t.id not in (select w.task.id from " + Work.class.getName() + " w where w.day.id = " + day.getId() + ") order by t.id desc");
		workTask.addListener(new ValueChangeListener(){

			@Override
			public void valueChange(ValueChangeEvent event) {
				Task task = (Task)event.getProperty().getValue();
				if(task == null)
					updateDays("select d from " + Day.class.getName() + " d order by d.day");
				else
					updateDays("select d from " + Day.class.getName() + " d where d.id not in (select w.day.id from " + Work.class.getName() + " w where w.task.id = " + task.getId() + ") order by d.day");
			}
			
		});
		hlTask.addComponent(workTask);
		hlTask.setExpandRatio(workTask, 5f);
		vl.addComponent(hlTask);
		
		HorizontalLayout hlHours = new HorizontalLayout();
		hlHours.setWidth("100%");
		Label lblHours = new Label("Hours:");
		lblHours.setWidth("100%");
		hlHours.addComponent(lblHours);
		hlHours.setExpandRatio(lblHours, 1f);
		workHours = new TextField();
		workHours.setWidth("100%");
		workHours.setValue(work.getHours());
		workHours.setImmediate(true);
		hlHours.addComponent(workHours);
		hlHours.setExpandRatio(workHours, 5f);
		vl.addComponent(hlHours);
		
		workResult = new RichTextArea("Result");
		workResult.setValue(work.getResult());
		workResult.setSizeFull();
		vl.addComponent(workResult);
		vl.setExpandRatio(workResult, 5f);
		
		HorizontalLayout hlOk = new HorizontalLayout();
		hlOk.setWidth("100%");
		Button okButton = new Button();
		switch(state){
		case CREATE:
			okButton.setCaption("Create");
			break;
		case EDIT:
			okButton.setCaption("Save");
			break;
		}
		okButton.addListener((ClickListener)this);
		hlOk.addComponent(okButton);
		hlOk.setComponentAlignment(okButton, Alignment.MIDDLE_RIGHT);
		vl.addComponent(hlOk);
	}
	
	private void updateDays(String jpql){
		if(workTask != null && workTask.getValue() != null && workDay != null && workDay.getValue() != null)
			return;
		workDay.removeAllItems();
		Query query = EM.createQuery(jpql);
		List list = query.getResultList();
		for(Day day: (List<Day>)list){
			workDay.addItem(day);
			Calendar currCal = Calendar.getInstance();
			currCal.set(Calendar.HOUR_OF_DAY, 0);
			currCal.set(Calendar.MINUTE, 0);
			currCal.set(Calendar.SECOND, 0);
			currCal.set(Calendar.MILLISECOND, 0);
			if(day.getDay().equals(currCal.getTime()))
				workDay.select(day);
		}
	}
	
	private void updateTasks(String jpql){
		if(workTask != null && workTask.getValue() != null && workDay != null && workDay.getValue() != null)
			return;
		workTask.removeAllItems();
		Query query = EM.createQuery(jpql);
		List list = query.getResultList();
		for(Task task: (List<Task>)list)
			workTask.addItem(task);
	}

	@Override
	public void buttonClick(ClickEvent event) {
		Object dayObj = workDay.getValue();
		Object taskObj = workTask.getValue();
		Object hoursObj = workHours.getValue();
		String result = workResult.getValue().toString();
		if (
				!(dayObj instanceof Day) 
				|| !(taskObj instanceof Task) 
				|| !(hoursObj.toString().matches("^[0-9]+[\\.]{0,1}[0-9]*"))
				) {
			getWindow().showNotification("Invalid information entered");
		} else {
			Day day = (Day)dayObj;
			Task task = (Task)taskObj;
			Float hours = Float.parseFloat(hoursObj.toString());
			work.setDay(day);
			work.setTask(task);
			work.setHours(hours);
			work.setResult(result);
			try{
				EM.getTransaction().begin();
				EM.persist(work);
				EM.getTransaction().commit();
				EM.clear();
			} finally {
				if(EM.getTransaction().isActive())
					EM.getTransaction().rollback();
			}
			getParent().removeWindow(this);
			myApp.getWorkTable().requestRepaint();
		}
	}

}
